#!/bin/sh
#
# Installation script for Readline, Python and python modules
# Author: Carlos Guirao
# Date: 23.05.03 
# $Id: ESO_PYTHON_INSTALL.sh,v 1.47 2009-02-18 13:46:30 cguirao Exp $
#
# 09.03.15 python2.5.1.5: python2.5.1-4 + pCFITSIO-0.99.3 + openssl-1.0.1c
#                         bsddb3-5.3.0 + db-4.7.25
# 18.02.09 python2.5.1-4: python2.5.1-3 + _md5,_sha,_sha256 and _sha512
# 09.01.08 python2.5.1-3: python2.5.1-2 + python-sybase-0.38
# 21.12.07 python2.5.1-2: python2.5.1.1 renamed python2.5.1-2
#           Requires RPMS sqlite-devel-3.1.2-3 and sqlite-3.1.2-3
# 14.11.07 python2.5.1.1: python2.5.1 + packages
# 07.11.05 python2.4.2.2: python2.4.2.1 + openssl-0.9.8d
# 07.11.05 python2.4.2.1: python2.4.2 +   all packages
# 26.10.05 python2.3.2.5: python2.3.2.4 + pcc-2_3_1.tar.gz
# 12.09.05 python2.3.2.4: python2.3.2.3 + db-4.3.28.tar.gz
# 14.04.05 python2.3.2.3: python2.3.2.2 + bsddb3-4.3.0.tar.gz
# 14.06.04 python2.3.2.2: python2.3.2.1 + qfits-4.3.7
# 13.05.04 python2.3.2.1: python2.3.2.0 + zlib-1.2.1
# 16.03.04 python2.3.2.0beta4 is renamed official python2.3.2.0
#          python2.3.2.0beta4: python2.3.2.0beta3 + bsddb3-4.1.6
#          python2.3.2.0beta3: python2.3.2.0beta2 + gdbm-1.8.3
#          python2.3.2.0beta2: python2.3.2.0beta1 + sybase-0.36
#          python2.3.2.0beta1: python2.3.2.0beta0 +Sybase 12.5 + RH9.0 patched
# 	   python2.3.2.0beta0: python2.3.2 with hacked Sybase 11.9 + RH9.0
# ----

packages="egenix-mx-base-2.0.6.tar.gz python-sybase-0.38.tar.gz numarray-1.4.1.tar.gz Numeric-24.0.tar.gz pCFITSIO-0.99.3.tar.gz qfits-5.4.0.tar.gz bsddb3-5.3.0.tar.gz"
#ready_packages="PCC_V1_1_Beta2.tar.gz NGAMS_V2_1_Beta3.tar.gz"
ready_packages="pcc-2_3_1.tar.gz"
readline=readline-5.0.tar.gz
gdbm=gdbm-1.8.3.tar.gz
openssl=openssl-1.0.1c.tar.gz

# Determine how to set prompt
if [ "`echo -n a`" = a ]; then
  # BSD echo
  NNN=-n
  CCC=
else
  # USG echo
  NNN=
  CCC='\c'
fi
readonly NNN CCC

#
# Print error message
#
error()
{
  echo '***' $* > /dev/tty
  false
}

#
#  A safe way to perform echo without sending a new-line
#
necho()
{
  echo $NNN "$*" $CCC > /dev/tty
}

#
#  Get a yes/no answer from the user; $1 is the prompt, $2 the default
#
ask_yn()
{
  prompt="$1"
  def="${2:-y}"           # yes is the default if not supplied
  while true; do
    necho "$prompt [$def]: "
    read answer
    answer="${answer:-$def}"
    case "$answer" in
      [yY])
        answer=y
        break
        ;;
      [nN])
        answer=n
        break
        ;;
      *)
        error Please enter "'y'" or "'n'".
        ;;
    esac
  done
  [ $answer = y ]
}

#
# Warn the installation if not done as root.
# In particular the installation of gdbm may fails if you try
# to install as another user than root, because it tries to
# change ownership to some files. This is the error:
#   /usr/bin/install: cannot change ownership of 
#   `.../python/util/include/gdbm.h': Operation not permitted
#
#if [ `whoami` != "root" ]; then
#  echo "Warning: The installation of package gdbm may fail if you"
#  echo "do not execute this installation as root"
#  if ask_yn "Do you want to continue?" "n"; then
#    echo "Ok. Let's continue..."
#  else
#    echo "Well done. Bye then.."
#    exit 1
#  fi
#fi
# CG. 18.08.04. This check is not necessary any more. gdbm is installed
# with the current UID:GID instead of the default bin:bin

#
# For each python release change the following definitions
# python_lib is the directory created under util/python/lib
# python_releas is our ESO distribution
#
python=Python-2.5.1.tgz
python_lib=python2.5	
python_release=python-2.5.1.5

cfitsio=cfitsio2510.tar.gz
expat=expat-1.95.8.tar.gz
#zlib=zlib-1.2.3.tar.gz
#db=db-4.3.29.tar.gz
zlib=zlib-1.2.5.tar.gz
db=db-4.7.25.tar.gz
#
# Commands:
# CG. On solaris 8 the tar does not work properly
# 
TAR=gtar

install_zlib()
{
# 
# Expat installation
#
echo "cd $tmpdir"
cd $tmpdir
package=$zlib
echo "======================================================================"
echo "Installing package=$package"
if [ ! -f $srcdir/$package ]; then
  echo "ERROR: Package $srcdir/$package does not exist"
  exit 1
fi
package_dir=`basename $package .tar.gz`
if [ -d $package_dir ]; then
  echo "Removing directory $package_dir"
  rm -rf $package_dir
fi
echo "gunzip -c $srcdir/$package | $TAR xf -"
gunzip -c $srcdir/$package | $TAR xf -
echo "cd $package_dir"
cd $package_dir
echo "./configure --prefix=${utildir} >install.log 2>&1"
./configure --prefix=${utildir} >install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
echo "make clean all install >>install.log 2>&1"
make clean all  install>>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
}

install_expat()
{
# 
# Expat installation
#
echo "cd $tmpdir"
cd $tmpdir
package=$expat
echo "======================================================================"
echo "Installing package=$package"
if [ ! -f $srcdir/$package ]; then
  echo "ERROR: Package $srcdir/$package does not exist"
  exit 1
fi
package_dir=`basename $package .tar.gz`
if [ -d $package_dir ]; then
  echo "Removing directory $package_dir"
  rm -rf $package_dir
fi
echo "gunzip -c $srcdir/$package | $TAR xf -"
gunzip -c $srcdir/$package | $TAR xf -
echo "cd $package_dir"
cd $package_dir

echo "make clean all >>install.log 2>&1"
make clean all >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
}

install_gdbm()
{
# 
# Gdbm installation
#
echo "cd $tmpdir"
cd $tmpdir
package=$gdbm
echo "======================================================================"
echo "Installing package=$package"
if [ ! -f $srcdir/$package ]; then
  echo "ERROR: Package $srcdir/$package does not exist"
  exit 1
fi
package_dir=`basename $package .tar.gz`
if [ -d $package_dir ]; then
  echo "Removing directory $package_dir"
  rm -rf $package_dir
fi
echo "gunzip -c $srcdir/$package | $TAR xf -"
gunzip -c $srcdir/$package | $TAR xf -
echo "cd $package_dir"
cd $package_dir
echo "./configure --prefix=${utildir} --disable-shared >install.log 2>&1"
./configure --prefix=${utildir} --disable-shared >install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
echo "make >>install.log 2>&1"
make >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
case $OS in
  Linux)
    ID=/usr/bin/id
    ;;
  HP-UX)
    ID=/usr/bin/id
    ;;
  SunOS)
    ID=/usr/xpg4/bin/id
    ;;
esac
export ID
USER_ID=`$ID -u $USER`
USER_GID=`$ID -g $USER`
echo "make BINOWN=$USER_ID BINGRP=$USER_GID install >>install.log 2>&1"
make BINOWN=$USER_ID BINGRP=$USER_GID install >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
}

install_readline()
{
# 
# Readline installation
#
echo "cd $tmpdir"
cd $tmpdir
package=$readline
echo "======================================================================"
echo "Installing package=$package"
if [ ! -f $srcdir/$package ]; then
  echo "ERROR: Package $srcdir/$package does not exist"
  exit 1
fi
package_dir=`basename $package .tar.gz`
if [ -d $package_dir ]; then
  echo "Removing directory $package_dir"
  rm -rf $package_dir
fi
echo "gunzip -c $srcdir/$package | $TAR xf -"
gunzip -c $srcdir/$package | $TAR xf -
echo "cd $package_dir"
cd $package_dir
echo "./configure --prefix=${utildir} --disable-shared >install.log 2>&1"
./configure --prefix=${utildir} --disable-shared >install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
echo "make >>install.log 2>&1"
make >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
echo "make install >>install.log 2>&1"
make install >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
}

install_openssl()
{
# 
# openssl installation
#
echo "cd $tmpdir"
cd $tmpdir
package=$openssl
echo "======================================================================"
echo "Installing package=$package"
if [ ! -f $srcdir/$package ]; then
  echo "ERROR: Package $srcdir/$package does not exist"
  exit 1
fi
package_dir=`basename $package .tar.gz`
if [ -d $package_dir ]; then
  echo "Removing directory $package_dir"
  rm -rf $package_dir
fi
echo "gunzip -c $srcdir/$package | $TAR xf -"
gunzip -c $srcdir/$package | $TAR xf -
echo "cd $package_dir"
cd $package_dir
echo "./config --prefix=${utildir}  >install.log 2>&1"
./config --prefix=${utildir} >install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
echo "make >>install.log 2>&1"
make >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
echo "make install >>install.log 2>&1"
make install >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
}

install_cfitsio()
{
# 
# CFITSIO installation
#
echo "cd $tmpdir"
cd $tmpdir
package=$cfitsio
echo "======================================================================"
echo "Installing package=$package"
if [ ! -f $srcdir/$package ]; then
  echo "ERROR: Package $srcdir/$package does not exist"
  exit 1
fi
package_dir=`basename $package .tar.gz`
if [ -d $package_dir ]; then
  echo "Removing directory $package_dir"
  rm -rf $package_dir
fi
echo "gunzip -c $srcdir/$package | $TAR xf -"
gunzip -c $srcdir/$package | $TAR xf -
echo "cd cfitsio"
cd cfitsio
echo "./configure >install.log 2>&1"
./configure >install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
echo "make >>install.log 2>&1"
make >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
}

install_db()
{
# 
# DB installation
#
echo "cd $tmpdir"
cd $tmpdir
package=$db
echo "======================================================================"
echo "Installing package=$package"
if [ ! -f $srcdir/$package ]; then
  echo "ERROR: Package $srcdir/$package does not exist"
  exit 1
fi
package_dir=`basename $package .tar.gz`
if [ -d $package_dir ]; then
  echo "Removing directory $package_dir"
  rm -rf $package_dir
fi
echo "gunzip -c $srcdir/$package | $TAR xf -"
gunzip -c $srcdir/$package | $TAR xf -
echo "cd $package_dir/build_unix"
cd $package_dir/build_unix
echo "../dist/configure --prefix=${utildir} >install.log 2>&1"
../dist/configure  --prefix=${utildir} --disable-shared CFLAGS=-fPIC >install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
echo "make >>install.log 2>&1"
make >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
echo "make install >>install.log 2>&1"
make install >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
}


install_python()
{
# 
# Python installation
#
echo "cd $tmpdir"
cd $tmpdir
package=$python
echo "======================================================================"
echo "Installing package=$package"
if [ ! -f $srcdir/$package ]; then
  echo "ERROR: Package $srcdir/$package does not exist"
  exit 1
fi
package_dir=`basename $package .tgz`
if [ -d $package_dir ]; then
  echo "Removing directory $package_dir"
  rm -rf $package_dir
fi
echo "gunzip -c $srcdir/$package | $TAR xf -"
gunzip -c $srcdir/$package | $TAR xf -
echo "cd $package_dir"
cd $package_dir
case $OS in
  Linux)
    set -x
    CFLAGS="-D_FILE_OFFSET_BITS=64" OPT="-g -O2 -D_FILE_OFFSET_BITS=64" ./configure --prefix=${utildir}/${python_release} >install.log 2>&1
    res=$?
    echo "_md5 md5module.c md5.c" >> Modules/Setup
    echo "_sha shamodule.c" >> Modules/Setup
    echo "_sha256 sha256module.c" >> Modules/Setup
    echo "_sha512 sha512module.c" >> Modules/Setup
    echo "readline readline.c -I${utildir}/include -L${utildir}/lib -lreadline -lncurses" >> Modules/Setup
    echo "gdbm gdbmmodule.c -I${utildir}/include -L${utildir}/lib -lgdbm" >> Modules/Setup
    echo "zlib zlibmodule.c -I${utildir}/include -L${utildir}/lib -lz" >> Modules/Setup
    echo "SSL definitions to Modules/Setup"
    cat << EOF >> Modules/Setup
SSL=$utildir
_ssl _ssl.c \\
	-DUSE_SSL -I\$(SSL)/include -I\$(SSL)/include/openssl \\
	-L\$(SSL)/lib -lssl -lcrypto
EOF
    ;;
  HP-UX)
    set -x
    OPT="-g -O2" ./configure --prefix=${utildir}/${python_release} >install.log 2>&1
    res=$?
    echo "readline readline.c -I${utildir}/include -L${utildir}/lib -lreadline -ltermcap" >> Modules/Setup
    echo "gdbm gdbmmodule.c -I${utildir}/include -L${utildir}/lib -lgdbm" >> Modules/Setup
#    expat_dir=`basename $expat .tar.gz`
#    echo "pyexpat pyexpat.c -I${tmpdir}/${expat_dir}/xmlparse -L${tmpdir}/${expat_dir}/xmlparse -lexpat" >> Modules/Setup
    echo "zlib zlibmodule.c -I${utildir}/include -L${utildir}/lib -lz" >> Modules/Setup
    echo "SSL definitions to Modules/Setup"
    cat << EOF >> Modules/Setup
SSL=$utildir
_ssl _ssl.c \\
	-DUSE_SSL -I\$(SSL)/include -I\$(SSL)/include/openssl \\
	-L\$(SSL)/lib -lssl -lcrypto
EOF
    ;;
  SunOS)
    set -x
    CFLAGS="-D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64" OPT="-g -O2 -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64" ./configure --prefix=${utildir}/${python_release} >install.log 2>&1
    res=$?
    echo "readline readline.c -I${utildir}/include -L${utildir}/lib -lreadline -ltermcap" >> Modules/Setup
    echo "gdbm gdbmmodule.c -I${utildir}/include -L${utildir}/lib -lgdbm" >> Modules/Setup
    echo "zlib zlibmodule.c -I${utildir}/include -L${utildir}/lib -lz" >> Modules/Setup
    echo "SSL definitions to Modules/Setup"
    cat << EOF >> Modules/Setup
SSL=$utildir
_ssl _ssl.c \\
	-DUSE_SSL -I\$(SSL)/include -I\$(SSL)/include/openssl \\
	-L\$(SSL)/lib -lssl -lcrypto
EOF
    ;;
esac
set +x

if [ $res != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
#
echo "make Modules/config.c" 
make Modules/config.c >>install.log 2>&1
echo "make Modules/readline.o" 
make Modules/readline.o >>install.log 2>&1
echo "make Modules/gdbmmodule.o" 
make Modules/gdbmmodule.o >>install.log 2>&1
echo "make Modules/zlibmodule.o" 
make Modules/zlibmodule.o >>install.log 2>&1
echo "make Modules/sha256module.o" 
make Modules/sha256module.o >>install.log 2>&1
echo "make Modules/sha512.o" 
make Modules/sha512.o >>install.log 2>&1
echo "make"
make >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROr: Installation of package $package failed"
  exit 1
fi
#
# Remake python with the new readline entries
#
echo "Remaking python with the readline"
echo "touch Modules/Setup; rm python; make python"
touch Modules/Setup; rm python; make python >>install.log 2>&1
echo "make install"
make install >>install.log 2>&1
if [ $? != 0 ]; then
  echo "ERROR: Installation of package $package failed"
  exit 1
fi
}

# 
# MAIN
#
echo "MAIN"
OS=`uname -s`
cd `dirname $0`
srcdir=`pwd`
pckg_dir=`dirname $srcdir`
utildir=$pckg_dir/util
tmpdir=$pckg_dir/tmp

#
# Clean the util and tmp directory 
#
rm -rf $utildir $tmpdir
mkdir $utildir $tmpdir

case $OS in
  Linux|SunOS)
    ;;
  HP-UX)
#    install_expat
    ;;
  *)
    echo "Unknown OS $OS"
    exit 1
    ;;
esac

install_readline
install_openssl
install_gdbm
install_cfitsio
install_zlib
install_db
install_python

#
# Setting this python the default one
#
set -x
cd $utildir
ln -s $python_release python
mkdir bin
cd bin
ln -s ../python/bin/python python
set +x

#
# Installation of Python modules
#
echo "cd $tmpdir"
cd $tmpdir
for package in $packages
do
 echo "======================================================================"
 if [ ! -f $srcdir/$package ]; then
   echo "ERROR: Package $srcdir/$package does not exist"
   continue 
 fi
 echo "Installing package=$package"
 package_dir=`basename $package .tar.gz`
 if [ -d $package_dir ]; then
    echo "Removing directory $package_dir"
    rm -rf $package_dir
 fi
 echo "gunzip -c $srcdir/$package | $TAR xf -"
 gunzip -c $srcdir/$package | $TAR xf -
 cd $package_dir 
 if [ -x ./configure ]; then
   echo "./configure >install.log 2>&1"
   ./configure >install.log 2>&1
   if [ $? != 0 ]; then
     echo "ERROR: Installation of package $package failed"
     exit 1
   fi
 fi
 if [ "$package_dir" = "bsddb3-5.3.0" ]; then 
   OPTIONS="--berkeley-db=$utildir"
 else
   OPTIONS=""
 fi
 
 echo "$utildir/bin/python setup.py $OPTIONS install"
 $utildir/bin/python setup.py $OPTIONS install >install.log 2>&1
 if [ $? != 0 ]; then
   echo "ERROR: Installation of package $package failed"
   continue
 fi
 # 
 # The pcfitsio.sl or pcfitsio.so have to be copied to 
 # ./util/python/lib/${python_lib}/site-packages
 #
 if [ -f pcfitsio.sl ]; then
   cp pcfitsio.sl $utildir/python/lib/${python_lib}/site-packages
 fi
 if [ -f pcfitsio.so ]; then
   cp pcfitsio.so $utildir/python/lib/${python_lib}/site-packages
 fi
 cd $tmpdir
done

#
# Installation of Python modules (already compiled)
#
echo "cd $utildir/python/lib/${python_lib}/site-packages"
cd $utildir/python/lib/${python_lib}/site-packages
for package in $ready_packages
do
 echo "======================================================================"
 if [ ! -f ${srcdir}/${package} ]; then
   echo "ERROR: Package $package does not exist"
   continue 
 fi
 echo "Installing package=$package"
 echo "gunzip -c ${srcdir}/${package} | $TAR xf -"
 gunzip -c ${srcdir}/${package} | $TAR xf -
done

#
# Finally run make install in the site-packages/ngmas/ngamsCClient
#
if [ -d $utildir/python/lib/${python_lib}/site-packages/ngams/ngamsCClient ]; then
 echo "======================================================================"
 cd $utildir/python/lib/${python_lib}/site-packages/ngams/ngamsCClient 
 echo "Compiling the ngamsCClient"
 make clean all >install.log 2>&1
 if [ $? != 0 ]; then
   echo "ERROR: Installation of ngamsCClient failed"
 fi
fi 

echo "END"
