#!/usr/bin/perl -wl

$version = '0.1';

# Include-file resides in same directory as program itself
push(@INC,$0 =~ /(.*\/)/);

require 'qfits.pl';

defined($qfits::version) || die;
defined($qfits::support) || die;

$help = "
  $qfits::prog version $version (using qfits.pl version $qfits::version).
  Print the md5sum of the Data in the FITS-files.

  Proto-type: Options may change in future versions.

Usage:

  $qfits::prog [options] 1.fits [2.fits [3.fits []]]

The file '-' is allowed once and is interpreted as standard input.

Options:
  -a        : Compute the md5sum of the hole file (incl. header)
  -verbose  : Print some extra information to stderr.
  -help     : Print this message to stderr and exit.

Example:

  $qfits::prog all_my*.{,t}fits

Bugs: With '-' the response time may be longer.

For further support contact $qfits::support

";

eval('use Digest::MD5');

$@ and die "$qfits::prog requires perl version better than $].
".$help;


@ARGV || die $help;

$verbose = 0;
$doall = 0;
for (;;) {

  $ARGV[0] eq '-a' and shift(), $doall = 1, next;
  $ARGV[0] eq '-verbose' and shift(), $verbose = 1, next;
  $ARGV[0] eq '-help' and warn($help), exit;

  last;

}

@ARGV || die $help;

for $file (@ARGV) {

  local($context);
  $context = new Digest::MD5;

  $context->reset();

  local(*IN);
  if ($file eq '-') {
    *IN = *STDIN;
  } else {
    open(IN,$file) or warn("error: cannot open file [$file]\n"), next;
  }

  local($ok);

  if ($doall) {

    $ok = 1;

    $context->addfile(IN);

  } else {

    $ok = 0;

    # Count number of extensions
    for ($mext = 0; ; $mext++) {

      &header(*IN,\@cards) || last;

      &split_cards(\@cards,\%vals,\%comments);
      %comments = ();

      local($size) = &data_size(\%vals);

      local($pos,$_);
      for ($pos = 0; $pos < $size; $pos += 2880) {

        read(IN,$_,2880) == 2880 or
          warn("Truncated header record at filepos=".tell(IN)."\n"), $ok = -1, last;

        $context->add($_);

      }

      $ok < 0 && last;

      $mext==0 and !defined($vals{EXTEND}) || $vals{EXTEND} ne 'T' and $ok = 1,
        last;

      eof(IN) and $ok = 1, last;

    }
  }

  if ($ok > 0) {
    print $context->hexdigest(), " $file\n";
  } else {
    print "$qfits::prog: file [$file] is not FITS\n";
  }

}

$verbose && warn(&run_stats);
